local util = require('util')

local item = {
  type = "item-with-entity-data",
  name = "accumulator-wagon",
  icon = "__accumulator-wagon-redux__/graphics/icons/accumulator-wagon.png",
  icon_size = 32,
  flags = {},
  subgroup = "transport",
  order = "a[train-system]-gz[accumulator-wagon]",
  place_result = "accumulator-wagon",
  stack_size = 5,
}

local altrec
if
settings.startup["accumulator-wagon-old-recipe"].value == false
then
altrec = 1
else
altrec = 0
end

local mulP = settings.startup["accumulator-wagon-power-multiplier"].value
local mulR = settings.startup["accumulator-wagon-recipe-multiplier"].value

local recipe = {
  type = "recipe",
  name = "accumulator-wagon",
  enabled = false,
  results = {{type = "item", name = "accumulator-wagon", amount = 1}},
}
  if altrec == 1 then
  recipe.ingredients =
  {
	{ type = "item", name = "iron-gear-wheel", amount = 10 },
	{ type = "item", name = "iron-plate", amount = 20},
	{ type = "item", name = "steel-plate", amount = 20},
	{ type = "item", name = "battery", amount = 5 * mulR},
	{ type = "item", name = "advanced-circuit", amount = math.floor ( 2.5 * mulR ) },
  }
  else
  recipe.ingredients =
  {
	{ type = "item", name = "cargo-wagon",amount =1 },
	{ type = "item", name = "accumulator", amount =1 * mulR },
	{ type = "item", name = "electronic-circuit", amount =math.floor ( 0.5 * mulR ) },
  }
  end

local technology = {
  type = "technology",
  name = "accumulator-wagon",
  icon = "__accumulator-wagon-redux__/graphics/technology/accumulator-wagon.png",
  icon_size = 256,
  icon_mipmaps = 4,
  effects = {
    {
      type = "unlock-recipe",
      recipe = "accumulator-wagon"
    }
  },
  unit = {
    count = 250,
    ingredients =
    {
      {"automation-science-pack", 1},
      {"logistic-science-pack", 1},
    },
    time = 30,
  },
  order = "c-k-d-z",
}
  if altrec == 1 then
  technology.prerequisites = 
  { "railway", "electric-energy-accumulators", "electronics" }
  else
  technology.prerequisites =
  { "railway", "electric-energy-accumulators" }
  end

local wagon = util.table.deepcopy(data.raw["cargo-wagon"]["cargo-wagon"])
wagon.name = "accumulator-wagon"
wagon.type = "fluid-wagon"
wagon.capacity = 25000
wagon.allow_passengers = false
wagon.color = {r = 0.1, g = 0.60, b = 0.1, a = 0.75}
wagon.icon = "__accumulator-wagon-redux__/graphics/icons/accumulator-wagon.png"
wagon.icon_size = 32
wagon.inventory_size = 0
wagon.horizontal_doors = nil
wagon.vertical_doors = nil
wagon.minable.result = "accumulator-wagon"

local passive_accumulator = util.table.deepcopy(data.raw["accumulator"]["accumulator"])
passive_accumulator.name = "accumulator-wagon-proxy-passive"
passive_accumulator.flags = {"not-upgradable","not-flammable", "not-deconstructable","not-repairable", "not-blueprintable", "not-deconstructable", "placeable-off-grid", "not-on-map", "placeable-neutral"}
passive_accumulator.order = "z[accumulator-wagon-proxy-passive]"
passive_accumulator.collision_mask = {layers= {["ghost"] = true}}
passive_accumulator.selectable_in_game = false
passive_accumulator.chargable_graphics.picture.layers[1].filename = "__accumulator-wagon-redux__/graphics/entity/transparent.png"
passive_accumulator.chargable_graphics.picture.layers[1].width=124
passive_accumulator.chargable_graphics.picture.layers[1].height=103
passive_accumulator.chargable_graphics.picture.layers[2] = nil
passive_accumulator.chargable_graphics.charge_animation.layers[1].layers[1].filename = "__accumulator-wagon-redux__/graphics/entity/transparent-charge.png"
passive_accumulator.chargable_graphics.charge_animation.layers[1].layers[2] = nil
passive_accumulator.chargable_graphics.charge_animation.layers[2] = nil
passive_accumulator.chargable_graphics.discharge_animation.layers[1].layers[1].filename = "__accumulator-wagon-redux__/graphics/entity/transparent-charge.png"
passive_accumulator.chargable_graphics.discharge_animation.layers[1].layers[2] = nil
passive_accumulator.chargable_graphics.discharge_animation.layers[2] = nil
passive_accumulator.energy_source.buffer_capacity = ( 5 * mulP ).."MJ"
-- / 25000 capacity = 0.2kJ * power multiplier per unit of battery fluid
passive_accumulator.energy_source.input_flow_limit = ( 0.3 * mulP ).."MW"
passive_accumulator.energy_source.output_flow_limit = ( 0.3 * mulP ).."MW"


local input_accumulator = util.table.deepcopy(passive_accumulator)
input_accumulator.name = "accumulator-wagon-proxy-input"
input_accumulator.type = "electric-energy-interface"
input_accumulator.order = "z[accumulator-wagon-proxy-input]"
input_accumulator.energy_source.usage_priority = "secondary-input"

local output_accumulator = util.table.deepcopy(passive_accumulator)
output_accumulator.name = "accumulator-wagon-proxy-output"
output_accumulator.type = "electric-energy-interface"
output_accumulator.order = "z[accumulator-wagon-proxy-output]"
output_accumulator.energy_source.usage_priority = "secondary-output"

if mods["space-age"] then -- quality
	for i=1, 5 do
		local input = table.deepcopy(input_accumulator)
		local output = table.deepcopy(output_accumulator)
		input.name = "accumulator-wagon-proxy-input-"..i
		output.name = "accumulator-wagon-proxy-output-"..i
		input.localised_name = {"entity-name.accumulator-wagon-proxy-input"}
		output.localised_name = {"entity-name.accumulator-wagon-proxy-output"}
		input.energy_source.input_flow_limit = ( (i+1) * 0.3 * mulP / (1+i*0.3) ).."MW" -- quality already buffs flow rates by 30% per level, but capacity by 100%
		output.energy_source.input_flow_limit = ( (i+1) * 0.3 * mulP / (1+i*0.3) ).."MW" -- negating the effect of quality and adding a +100% multiplier
		input.energy_source.output_flow_limit = ( (i+1) * 0.3 * mulP / (1+i*0.3) ).."MW"
		output.energy_source.output_flow_limit = ( (i+1) * 0.3 * mulP / (1+i*0.3) ).."MW"
		--input.energy_source.buffer_capacity = ( (i+1) * 5 * mulP ).."MJ"    -- applied in control script, so it supports higher quality than 5
		--output.energy_source.buffer_capacity = ( (i+1) * 5 * mulP ).."MJ"
		data:extend{input,output}
	end
end

local fluid = {
  type = "fluid",
  name = "battery-fluid",
  auto_barrel = false,
  default_temperature = 15,
  heat_capacity = "200J",
  base_color = {r=0.18, g=0.82, b=0.18},
  flow_color = {r=0.5, g=0.5, b=0.5},
  max_temperature = 100,
  icon = "__accumulator-wagon-redux__/graphics/icons/battery-fluid.png",
  icon_size = 32,
  pressure_to_speed_ratio = 0.4,
  flow_to_energy_ratio = 0.59,
  order = "a[fluid]-z[battery-fluid]"
}

local signal_group = {
  type = "item-subgroup",
  name = "accumulator-wagon-signal",
  group = "signals",
  order = "z[accumulator-wagon-signal]"
}

local charge_signal = {
  type = "virtual-signal",
  name = "accumulator-wagon-charge",
  icon = "__accumulator-wagon-redux__/graphics/icons/accumulator-wagon-charge.png",
  icon_size = 32,
  subgroup = "accumulator-wagon-signal",
  order = "a[special]-a[accumulator-wagon-charge]"
}

local discharge_signal = {
  type = "virtual-signal",
  name = "accumulator-wagon-discharge",
  icon = "__accumulator-wagon-redux__/graphics/icons/accumulator-wagon-discharge.png",
  icon_size = 32,
  subgroup = "accumulator-wagon-signal",
  order = "a[special]-b[accumulator-wagon-discharge]"
}

data:extend({ item, recipe, technology, wagon, passive_accumulator, input_accumulator, output_accumulator, fluid, signal_group, charge_signal, discharge_signal })
